%%
%  Compute a Cascadia subduction zone slip deficit model
%    Jefff Freymueller, Univerrity of Alaska Fairbanks
%    jfreymueller@alaska.edu
%
%  Data used here are the observations of McCaffrey et al. (2013) with the
%  block model rotation subtracted.
%
% Citation:
%      McCaffrey, R., R. W. King, S. J. Payne, and M. Lancaster (2013),
%      Active tectonics of northwestern U.S. inferred from GPS-derived
%      surface velocities, J. Geophys. Res. Solid Earth, 118, doi:10.1029/2012JB009473.
%
%
% Note -- this code is not super-efficient because it is not vectorized.

clear ; clc; close all;

%%
% Parameters to choose

% Profile origin (a point along the trench), azimuth, and width (km)

origin_lat = YOU_CHOOSE;
origin_lon = YOU_CHOOSE;
az = YOU_CHOOSE;
profwidth = YOU_CHOOSE;

% Plate motion rate and azimuth (from McCaffrey et al. (2007), Juan de Fuca
%   plate relative to crustal blocks
% Un-comment the most appropriate of these:

% Vancouver Island
plate_vel = 46;
plate_az = 54;

% Oregon-Washington border
plate_vel = 36;
plate_az = 61;

% Central Oregon
plate_vel = 35;
plate_az = 59;

%%
% Fault model parameters to adjust

dip  = YOU_CHOOSE;        % dip angle
depth = 5;                % depth at trench
width = YOU_CHOOSE;       % width of locked region along fault plane (km)
locking = YOU_CHOOSE;     % locking fraction (0-1)


%%
%   Calculate the fault model

deg2r = pi/180;

llh_origin = [origin_lat ; origin_lon ; 0];

% Load the data file

file = 'McCaffrey_Cascadia_minus_rot.txt';
fid = fopen(file, 'r');
C = textscan(fid, '%s %f %f %f %f %f %f');
staname = C{1};
llh = [C{3}, C{2}, zeros(size(C{3}))]';
enu = [C{4}, C{5}, zeros(size(C{4}))]';
%enusig = [C{6}; C{7} ; ones(size(C{4}))];

[dist, trans, profvel] = project_profile(llh_origin, az, width, llh, enu);
dist  = dist/1000;
trans = trans/1000;
nsta  = length(dist);
profsig = ones(size(profvel));
cov = diag(profsig(:), 0);

% Poisson's Ratio
nu = 0.25;

%   Define Fault model
nf = 1;

%
dip = -1*dip;   % trick so that we can specify the top instead of bottom of fault plane
Vp  = plate_vel;

ang = -1*(az - plate_az);   % angle from profile to plate motion, positive clockwise
angr = ang*deg2r;

ss = locking*Vp*sin(angr);
ds = locking*Vp*cos(angr);
op = 0;


strik = 180.0;    % fault extends to the left of the strike line
len = 5000.0;
delX = 0;
delY = 0;
%delY = -len/2;
%
dis_geom(1,:) = [len,width,depth,dip,strik,delX,delY,ss,ds,op];
%
% If we use a second fault segment
%  set nf = 2 and then add the parameters for the second fault segment:
%dis_geom(2,:) = [len,wid,dep,dip,strik,delE,delN,ss,ds,op];
%
% The rest of the code handles multiple fault segments

%   Compute model at data locations

u = zeros(3,nsta);
for i= 1:nsta,
   uu = disloc(nu, dis_geom, [dist(i), 0.0]);
   u(1,i)  = u(1,i) + uu(1);
   u(2,i)  = u(2,i) + uu(2);
   u(3,i)  = u(3,i) + uu(3);
end


%
%    Compare observed to model
%
resid = profvel - u;
r = zeros(3*nsta,1);
for i = 1:nsta
   k = 3*(i - 1);
   r(k+1:k+3) = resid(1:3,i);
end
% 
chi2 = r'*inv(cov)*r
dof = 3*nsta - 6;
chi2dof = chi2/dof

%   Make a plot of model with data

xmin = 50.00; xmax = 600.00;
x = linspace(xmin, xmax,  250);
m = length(x);

%initialize plot arrays to zero
uz = zeros(1,m); ux = zeros(1,m);  uy=zeros(1,m);
%
for i= 1:m,
    uu = disloc(nu, dis_geom, [x(i), 0.0]);
    ux(i)  = ux(i) + uu(1);
    uy(i)  = uy(i) + uu(2);
    uz(i)  = uz(i) + uu(3);
end

%%
%  Make a plot

figure
orient tall;
%
subplot(311), 
plot([0 600], [0 0], 'k--');   % plot dashed line at zero
hold on;
plot(x, ux, 'r-', 'LineWidth', 2);
title('Cascadia');
errorbar(dist, profvel(1,:)', profsig(1,:)', 'bd');
%plot(dist, u(1,:)', 'ro');   % plot model values at data locations as red circles
xlabel('Distance (km)');
ylabel('Trench-normal Velocity (m/yr)');

subplot(312),
plot([0 600], [0 0], 'k--');   % plot dashed line at zero
hold on;
plot(x, uy, 'r-', 'LineWidth', 2);
errorbar(dist, profvel(2,:)', profsig(2,:)', 'bd'),
%plot(dist, u(2,:)', 'ro');    % plot model values at data locations as red circles
xlabel('Distance (km)')
ylabel('Trench-parallel Velocity (mm/yr)')

subplot(313),
plot([0 600], [0 0], 'k--');   % plot dashed line at zero
hold on;
plot(x, uz, 'r-', 'LineWidth', 2);
xlabel('Distance (km)')
ylabel('Vertical Velocity (mm/yr)')

hold off
