function [u, e] = okada(alp, xrec, source)
%[u e] = okada(alp, xrec, source)
%
%  Evaluate dislocation for rectangular fault in elastic halfspace
%
%  Inputs:
%     alp    : medium constant = mu/(lambda+mu)
%     xrec   : site coordinates
%     source : vector (10 by n) with fault parameters
%              (if source is one-dimensional it can be 10 by 1 or 1 by 10)
%        1. dep   = depth
%        2. al1   = fault length 1
%        3. al2   = fault length 2
%        4. aw1   = fault width 1
%        5. aw2   = fault width 2
%        6. sd    = sin of dip angle
%        7. cd    = cos of dip angle
%        8. disl1 = strike-slip dislocation magnitude
%        9. disl2 = dip-slip dislocation magnitude
%       10. disl3 = tensile (opening) dislocation magnitude
%
%  Outputs:
%     u (3,1) : displacement vector (E, N, U)
%     e (3,3) : strain tensor (symmetric)
%        e(3,1), e(3,2) tilts
%
%  M-file version of Y. Okada's fortran code
%  Probably this could be made much more efficient, but this is a first
%  cut at it.
%

u = zeros(3,1); u1 = u;
e = zeros(3,3); e1 = e;

%  Get size of source

[n m] = size(source);

if (n == 1) 
   if (m == 10)
      source = source';
   else
      error('okada: Variable source must have 10 by N elements');
   end
end

for i = 1:m,

%     Extract source information

   dep   = source(1,i);
   al1   = source(2,i);
   al2   = source(3,i);
   aw1   = source(4,i);
   aw2   = source(5,i);
   sd    = source(6,i);
   cd    = source(7,i);
   disl1 = source(8,i);
   disl2 = source(9,i);
   disl3 = source(10,i);

   x = xrec(1);
   y = xrec(2);

%     P and Q are useful intermediate values

   p = y*cd + dep*sd;
   q = y*sd - dep*cd;

%   Call srectg (indefinite integral) four time, one for each corner

   [u1, e1] = srectg(alp,x-al1,p-aw1,q,sd,cd,disl1,disl2,disl3);
   u = u + u1; e = e + e1;
   [u1, e1] = srectg(alp,x-al2,p-aw1,q,sd,cd,disl1,disl2,disl3);
   u = u - u1; e = e - e1;
   [u1, e1] =  srectg(alp,x-al1,p-aw2,q,sd,cd,disl1,disl2,disl3);
   u = u - u1; e = e - e1;
   [u1, e1] =  srectg(alp,x-al2,p-aw2,q,sd,cd,disl1,disl2,disl3);
   u = u + u1; e = e + e1;

end


%  srectg is called by okada. srectg is the indefinite integral

function [u, e] = srectg(alp,xi,et,q,sd,cd,disl1,disl2,disl3)
%[u e] = srectg(alp,xi,et,q,sd,cd,disl1,disl2,disl3)
%
%  Evaluate indefinite integral for rectangular fault in elastic halfspace
%

u = zeros(3,1);
e = zeros(3,3);

%  Some useful intermediate variables

xi2 = xi*xi;
et2 = et*et;
q2 = q*q;
r2 = xi2 + et2 + q2;
r = sqrt(r2);
r3 = r*r2;
d = et*sd - q*cd;
y = et*cd + q*sd;
ret = r + et;
if ( ret < 0 )
   ret = 0;
end
rd = r + d;
rrd = 1/(r*rd);

% More useful intermediate variables, derived from the above

if ( q ~= 0)
   tt = atan( xi*et/(q*r) );
else
   tt = 0;
end

if ( ret ~= 0 )
   re = 1/ret;
   dle=log(ret);
else
   re = 0;
   dle=-log(r-et);
end

rrx = 1/(r*(r+xi));
rre = re/r;
axi = (2*r+xi)*rrx*rrx/r;
aet = (2*r+et)*rre*rre/r;


%  Yet more useful intermediate variables.

if (cd ~= 0)
%   ==============================
%   =====   inclined fault   =====
%   ==============================

   td = sd/cd;
   x = sqrt(xi2+q2);
   if (xi == 0)
      a5 = 0;
   else
      a5 = alp*2/cd*atan( (et*(x+q*cd)+x*(r+x)*sd) / (xi*(r+x)*cd) );
   end
   a4 = alp/cd*( log(rd) - sd*dle );
   a3 = alp*(y/rd/cd - dle) + td*a4;
   a1 =-alp/cd*xi/rd        - td*a5;
   c1 = alp/cd*xi*(rrd - sd*rre);
   c3 = alp/cd*(q*rre - y*rrd);
   b1 = alp/cd*(xi2*rrd - 1)/rd - td*c3;
   b2 = alp/cd*xi*y*rrd/rd       - td*c1;
else
%   ==============================
%   =====   vertical fault   =====
%   ==============================

   rd2 = rd*rd;
   a1 =-alp/2*xi*q/rd2;
   a3 = alp/2*( et/rd + y*q/rd2 - dle );
   a4 =-alp*q/rd;
   a5 =-alp*xi*sd/rd;
   b1 = alp/2*  q  /rd2*(2*xi2*rrd - 1);
   b2= alp/2*xi*sd/rd2*(2*q2 *rrd - 1);
   c1 = alp*xi*q*rrd/rd;
   c3 = alp*sd/rd*(xi2*rrd - 1);
end

a2 = -alp*dle - a3;
b3 = -alp*xi*rre - b2;
b4 = -alp*( cd/r + q*sd*rre ) - b1;
c2 =  alp*(-sd/r + q*cd*rre ) - c3;


%  Now evealuate the integral at log last. Separate terms for the strike-slip,
%  dip-slip and tensile contributions


%    ======================================
%    =====  strike-slip contribution  =====
%    ======================================
if (disl1 ~= 0)

   un = disl1/(2*pi);
   req = rre*q;

   u(1) = u(1) - un*( req*xi +   tt    + a1*sd );
   u(2) = u(2) - un*( req*y  + q*cd*re + a2*sd );
   u(3) = u(3) - un*( req*d  + q*sd*re + a4*sd );

   e(1,1) = e(1,1) + un*( xi2*q*aet - b1*sd );
   e(1,2) = e(1,2) + un*( xi2*xi*( d/(et2+q2)/r3 - aet*sd ) - b2*sd );
   e(2,1) = e(2,1) + un*( xi*q/r3*cd + (xi*q2*aet - b2)*sd );
   e(2,2) = e(2,2) + un*( y *q/r3*cd + (q*sd*(q2*aet-2*rre) ...
                                  - (xi2+et2)/r3*cd - b4)*sd );
   e(3,1) = e(3,1) + un*(-xi*q2*aet*cd + (xi*q/r3 - c1)*sd );
   e(3,2) = e(3,2) + un*( d*q/r3*cd + (xi2*q*aet*cd - sd/r + y*q/r3 - c2)*sd );

end


%    ===================================
%    =====  dip-slip contribution  =====
%    ===================================
if (disl2 ~= 0)

   un = disl2/(2*pi);
   sdcd = sd*cd;

   u(1) = u(1) - un*( q/r             - a3*sdcd );
   u(2) = u(2) - un*( y*q*rrx + cd*tt - a1*sdcd );
   u(3) = u(3) - un*( d*q*rrx + sd*tt - a5*sdcd );

   e(1,1) = e(1,1) + un*( xi*q/r3            + b3*sdcd );
   e(1,2) = e(1,2) + un*( y *q/r3 - sd/r     + b1*sdcd );
   e(2,1) = e(2,1) + un*( y *q/r3 + q*cd*rre + b1*sdcd );
   e(2,2) = e(2,2) + un*( y*y*q*axi - (2*y*rrx + xi*cd*rre)*sd + b2*sdcd );
   e(3,1) = e(3,1) + un*( d *q/r3 + q*sd*rre + c3*sdcd );
   e(3,2) = e(3,2) + un*( y*d*q*axi - (2*d*rrx + xi*sd*rre)*sd + c1*sdcd );

end


%    ========================================
%    =====  tensile-fault contribution  =====
%    ========================================
if (disl3 ~= 0)

   un = disl3/(2*pi);
   sdsd = sd*sd;

   u(1) = u(1) + un*( q2*rre                       - a3*sdsd );
   u(2) = u(2) + un*(-d*q*rrx - sd*(xi*q*rre - tt) - a1*sdsd );
   u(3) = u(3) + un*( y*q*rrx + cd*(xi*q*rre - tt) - a5*sdsd );

   e(1,1) = e(1,1) - un*( xi*q2*aet             + b3*sdsd );
   e(1,2) = e(1,2) - un*(-d*q/r3 - xi2*q*aet*sd + b1*sdsd );
   e(2,1) = e(2,1) - un*( q2*(cd/r3 + q*aet*sd) + b1*sdsd );
   e(2,2) = e(2,2) - un*((y*cd-d*sd)*q2*axi - 2*q*sd*cd*rrx ...
                            - (xi*q2*aet - b2)*sdsd );
   e(3,1) = e(3,1) - un*( q2*(sd/r3 - q*aet*cd) + c3*sdsd );
   e(3,2) = e(3,2) - un*((y*sd+d*cd)*q2*axi + xi*q2*aet*sd*cd ...
                             - (2*q*rrx - c1)*sdsd );

end


%  Fill in symmetric parts

e(1,2) = e(2,1);
e(1,3) = e(3,1);
e(2,3) = e(3,2);
