/*
 * Copyright (C) Sergey Kolesov 2012-2014 <kolesov@ocean.phys.msu.ru>
 * See ffaultfdisp.cpp for licensing details
 */

#ifdef _OPENMP
#include <omp.h>
#endif
#include <sstream>
#include <iostream>
#include <cstdlib>
#include <iomanip>

#include "solver.h"
//===============================
double LeftLon=0,RightLon=0,BottomLat=0,TopLat=0,LonStep=1./60,LatStep=1./60;   //  Grid region & intervals
int Nlon,Nlat; // Grid dimensions
vector<TFault> FaultPlane;   //  matrix of subfaults
vector<double> DispX,DispY,DispZ; //  bottom displacement
vector<double> Lons,Lats; // Set of points' coordinates
vector<string> Comments; // Comments on points
bool fRes2Cout=true; // Results to stdout flag
//===============================
void CheckLon(double& Lon1,double& Lon2)
{
 if(Lon1 == Lon2) throw "Equal longitudes in a region";
 if(Lon1 > Lon2) swap(Lon1,Lon2);
 if(Lon2-Lon1 > 180) cout << "WRN: Huge longitude span: " << Lon2-Lon1 << " deg"<< endl; // Warn if longitude span is more than 180 deg
}
//===============================
void CheckLat(double& Lat1, double& Lat2)
{
 if(Lat1 == Lat2) throw "Equal latitudes in a region";
 if(Lat1 < -90 || Lat2 < -90 || Lat1 > 90 || Lat2 > 90) throw "Latitude is definitely wrong";
 if(Lat1 > Lat2) swap(Lat1,Lat2);
 if(Lat2-Lat1 > 90)  cout << "WRN: Huge latitude span: " << Lat2-Lat1 << " deg" << endl; // Warn if latitude span is more than 90 deg
}
//===============================
void SetGridBounds(char* Str)
{
stringstream str(Str);
 str >> LeftLon;
 str.ignore(255,'/');
 str >> BottomLat;
 str.ignore(255,'/');
 str >> RightLon;
 str.ignore(255,'/');
 str >> TopLat;
 if(str.fail()) throw "Syntax error in region specification";
 CheckLon(LeftLon,RightLon);
 CheckLat(BottomLat,TopLat);
}
//===============================
void SetGridInterval(char* Str)
{
stringstream str(Str);
 str >> LonStep;
 str.ignore(255,'/');
 str >> LatStep;
 if(str.fail()) throw "Syntax error in grid interval specification";
 if(LonStep <= 0 || LonStep > 60 || LatStep <= 0 || LatStep > 60) throw "Grid interval is negative or more than 1 deg"; //  More than 1 deg step is treated as typo

 LonStep /= 60; LatStep /= 60;
}
//===============================
void SetRes2Cout(bool O_RLY)
{
 fRes2Cout = O_RLY;
}
//===============================
void SetCR(char* Str)
{
stringstream str(Str);
 str >> TFault::CR;
 if(str.fail()) throw "Wrong CR value specified";
 if(TFault::CR <= 0 || TFault::CR > 10) throw "CR is zero, negative or exceeds 10";   //  Sane CR interval
}
//===============================
void SetFaultPosition(char* Str)
{
stringstream str(Str);
int fp;
 str >> fp;
 if(str.fail()) throw  "Syntax error in fault centroid position specification";
 if(fp < 0 || fp > 3)  throw  "Fault centroid position should be from 0 to 3 (see help)";

 TFault::FaultPos = TFaultPosition(fp);
}
//===============================
void ParseFiniteFault(istream& Is)
{
string line="";
int ln=0;
 Is.exceptions(istream::failbit | istream::badbit | istream::eofbit);
 Is >> setw(255);   //   Get 255 chars at max

 try
 {
  do
  {
   ln++;
   getline(Is,line);

int c = line.find_first_of("#\r");   // Eliminating comments and CRs
   if(c != string::npos) line=line.substr(0,c);
   c = line.find_first_not_of(" \t"); // Skip empty strings
   if(c == string::npos) continue;

   CreateFault(line); // Creating fault from string
  } while(1);
 }
 catch(istream::failure& e)
 {
  if(Is.eof()) return;
  cerr << "ERR: Something went wrong while reading input (line " << ln << ")" << endl;
  exit(1);
 }
 catch(string& what)
 {
  cerr << "ERR: Failed to parse input data:\"" << what << "\" (line " << ln << ")" << endl;
  exit(1);
 }
}
//===============================
void CreateFault(string& Str)
{
stringstream stream(Str);
double Lat,Lon,Depth,Slip,Rake,Strike,Dip,L,W;
/*
  Finite-fault file format:
Lat Lon depth[km] slip[cm] rake strike dip L[m] W[m]
*/
 stream >> Lat >> Lon >> Depth >> Slip >> Rake >> Strike >> Dip >> L >> W;
 if(stream.fail()) throw Str;

TFault fault(Lon,Lat,Depth,Slip,Rake,Strike,Dip,L,W);
 FaultPlane.push_back(fault);
}
//===============================
void MakeGrid(void)
{
 Nlon = floor((RightLon-LeftLon)/LonStep+0.5) + 1;
 Nlat = floor((TopLat-BottomLat)/LatStep+0.5) + 1;

 DispX.resize(Nlon*Nlat,0);
 DispY.resize(Nlon*Nlat,0);
 DispZ.resize(Nlon*Nlat,0);

 for(int f=0;f<FaultPlane.size();f++)
 {
int nlat;
#pragma omp parallel for shared(nlat,DispX,DispY,DispZ)
  for(nlat=0;nlat<Nlat;nlat++)
  {
const double lat = BottomLat+nlat*LatStep;
   for(int nlon=0;nlon<Nlon;nlon++)
   {
const double lon = LeftLon+nlon*LonStep;
double ue,un,uz;
    FaultPlane[f].CalcDisp(lon,lat,&ue,&un,&uz);

    DispX[nlat*Nlon+nlon] += ue;
    DispY[nlat*Nlon+nlon] += un;
    DispZ[nlat*Nlon+nlon] += uz;
   }
  }

  if(!fRes2Cout) cout << "Fault " << f+1 << " of " << FaultPlane.size() << '\r' << flush;
 }
 if(!fRes2Cout) cout << endl;
}
//===============================
void SaveGrid(ostream& Os)
{
 for(int nlat=0;nlat<Nlat;nlat++)
  for(int nlon=0;nlon<Nlon;nlon++)
  {
   Os.unsetf(ios_base::fixed);
   Os << setprecision(8) << LeftLon+nlon*LonStep << '\t' << BottomLat+nlat*LatStep << '\t';
   Os.setf(ios_base::fixed);
   Os << setprecision(5) << DispX[nlat*Nlon+nlon] << '\t' << DispY[nlat*Nlon+nlon] << '\t' << DispZ[nlat*Nlon+nlon] << '\n';
  }
}
//===============================
void ParseSet(istream& Is)
{
string line="";
int ln=0;
double lon,lat;
string comment;

 Is >> setw(255);   //   Get 255 chars at max
 do
 {
  ln++;
  getline(Is,line);
  if(Is.eof()) break;
stringstream Str(line);
  /*
   Points set file format:
   Lon  Lat  Comment-till-the-end-of-line
  */
  Str >> lon >> lat;
  getline(Str,comment);
  if(Str.fail()) cerr << "WRN: Failed to parse point:\"" << line << "\" (line " << ln << ")" << endl;
  else
  {
   Lons.push_back(lon);
   Lats.push_back(lat);
   Comments.push_back(comment);
  }
 } while(1);
}
//===============================
void CalcSet(void)
{
 DispX.resize(Lons.size(),0);
 DispY.resize(Lons.size(),0);
 DispZ.resize(Lons.size(),0);

 for(int i=0;i<Lons.size();i++)
 {
int f;
#pragma omp parallel for shared(f,DispX,DispY,DispZ)
  for(f=0;f<FaultPlane.size();f++)
  {
double ue,un,uz;
   FaultPlane[f].CalcDisp(Lons[i],Lats[i],&ue,&un,&uz);
#pragma omp atomic
    DispX[i] += ue;
#pragma omp atomic
    DispY[i] += un;
#pragma omp atomic
    DispZ[i] += uz;
  }

  if(!fRes2Cout) cout << "Point " << i+1 << " of " << Lons.size() << '\r' << flush;
 }
 if(!fRes2Cout) cout << endl;
}
//===============================
void SaveSet(ostream& Os)
{
 for(int i=0;i<Lons.size();i++)
 {
  Os.unsetf(ios_base::fixed);
  Os << setprecision(8) << Lons[i] << '\t' << Lats[i] << '\t';
  Os.setf(ios_base::fixed);
  Os << setprecision(5) << DispX[i] << '\t' << DispY[i] << '\t' << DispZ[i] << '\t' << Comments[i] << '\n';
 }
}
//===============================